package cn.com.qmth.examcloud.bridge.modules.cloudmarking.controller;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.HttpClientUtil;
import cn.com.qmth.examcloud.commons.util.IOUtil;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.RegExpUtil;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamRecordCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamRecordQuestionsCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.OeExamStudentCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ToBeMarkExamRecordBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ToBeMarkExamStudentBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ToBeMarkSubjectiveAnswerBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.GetToBeMarkExamStudentReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.GetToBeMarkExamRecordResp;
import cn.com.qmth.examcloud.core.oe.admin.api.response.GetToBeMarkExamStudentResp;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import com.google.common.collect.Maps;
import com.mysql.cj.util.StringUtils;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Description 云阅卷客户端接口
 * 错误码范围(102001 至 101999)
 * @Author lideyin
 * @Date 2019/11/20 14:49
 * @Version 1.0
 */
@RestController
@RequestMapping("cmcClient")
public class CloudMarkingClientController {
    @Autowired
    ExamRecordQuestionsCloudService examRecordQuestionsCloudService;

    @Autowired
    SystemProperties systemConfig;

    @Autowired
    ExamRecordCloudService examRecordCloudService;

    @Autowired
    OeExamStudentCloudService oeExamStudentCloudService;

    Logger logger = LoggerFactory.getLogger(CloudMarkingClientController.class);

    /**
     * 批量生成主观题答案，以压缩文件方式返回
     */
    @GetMapping(value = "batchGetAnswer/{examId}")
    public ResponseEntity<?> batchGetSubjectiveAnswer(@PathVariable Long examId) {
        if (null == examId) {
            throw new StatusException("101001", "考试批次id不允许为空");
        }

        GetToBeMarkExamStudentReq req = new GetToBeMarkExamStudentReq();
        req.setExamId(examId);
        //获取考试批次下所有考生
        GetToBeMarkExamStudentResp toBeMarkExamStudentResp = oeExamStudentCloudService.getToBeMarkExamStudent(req);
        List<ToBeMarkExamStudentBean> examStudentList = toBeMarkExamStudentResp.getExamStudentList();
        if (examStudentList.isEmpty()) {
            throw new StatusException("101002", "该考试批次下无待阅考生数据");
        }

        for (ToBeMarkExamStudentBean stu : examStudentList) {
            String hostUrl = "http://localhost:8020/cmc/getSubjectiveAnswer/" + stu.getExamStudentId();
            HttpGet httpGet = HttpClientUtil.buildHttpGet(hostUrl);
            CloseableHttpClient httpClient = HttpClientUtil.getHttpClient();
            CloseableHttpResponse response = null;

            try {
                response = httpClient.execute(httpGet);
                int statusCode = response.getStatusLine().getStatusCode();
                String strEntity = EntityUtils.toString(response.getEntity(), "UTF-8");
                if (HttpStatus.SC_OK != statusCode) {
                    logger.error("[HTTP-FAIL]. statusCode=" + statusCode + "; responseEntity="
                            + strEntity + "; uri=" + httpGet.getURI());

                    continue;
                }
                saveDataToLocal(strEntity, examId);

            } catch (IOException e) {
                throw new StatusException("IO操作异常", e.getMessage());
            } finally {
                if (null != response) {
                    HttpClientUtil.close(response);
                }
            }
        }

        return ResponseEntity.ok().body("数据生成完毕.");
    }

    private void saveDataToLocal(String strEntity, Long examId) {
        GetToBeMarkExamRecordResp resp = JsonUtil.fromJson(strEntity, GetToBeMarkExamRecordResp.class);

        if (!resp.getToBeMarkExamRecordBeanList().isEmpty()) {

            StringBuilder sb = new StringBuilder();
            String COL_SEPARATOR = "\t";//列分隔符
            String LINE_BREAK = "\r\n";//换行
            //循环待审阅的考试记录
            for (ToBeMarkExamRecordBean rb : resp.getToBeMarkExamRecordBeanList()) {
                //构建文本文件串
                sb.append(rb.getStudentName()).append(COL_SEPARATOR)
                        .append(rb.getStudentCode()).append(COL_SEPARATOR)
                        .append(rb.getCourseName()).append(COL_SEPARATOR)
                        .append(rb.getCourseCode()).append(COL_SEPARATOR)
                        .append(rb.getPaperType()).append(COL_SEPARATOR)
                        .append(rb.getExamRecordDataId()).append(LINE_BREAK);

                //答案临时存放目录
                final String tempAnswerDir =
                        systemConfig.getTempDataDir() + "/exam-" + rb.getExamId() + "/" + rb.getCourseCode() + "-" + rb.getPaperType();

                List<Map<String, Object>> answerMapList = new ArrayList<>();

                //循环主观题答案
                for (ToBeMarkSubjectiveAnswerBean sab : rb.getSubjectiveAnswerList()) {
                    Map<String, Object> answerMap = Maps.newHashMap();
                    answerMap.put("mainNumber", sab.getMainNumber());
                    answerMap.put("subNumber", sab.getOrder());

                    Map<String, Object> bodyMap = Maps.newHashMap();

                    List<Map<String, Object>> sectionMapList = new ArrayList<>();
                    Map<String, Object> sectionMap = Maps.newHashMap();
                    List<Map<String, Object>> blockMapList = new ArrayList<>();

                    //校验答案格式，校验成功才追加至集合，否则不追加
                    if (validateAnswer(sab.getRealAnswerType(), sab.getStudentAnswer())) {
                        //图片文件特殊处理
                        if (!StringUtils.isNullOrEmpty(sab.getStudentAnswer()) &&
                                "image".equals(sab.getRealAnswerType())) {
                            if (sab.getStudentAnswer().indexOf("|") > -1) {
                                String[] imgAnswers = sab.getStudentAnswer().split("\\|");
                                for (int i = 0; i < imgAnswers.length; i++) {
                                    Map<String, Object> blockMap = Maps.newHashMap();
                                    blockMap.put("type", sab.getRealAnswerType());
                                    blockMap.put("value", transformedAnswer(sab.getRealAnswerType(), imgAnswers[i]));

                                    //又拍云图片的宽*高
                                    String imgWH = RegExpUtil.find(imgAnswers[i], "(\\d+)x(\\d+)");
                                    if (!StringUtils.isNullOrEmpty(imgWH)) {
                                        Map<String, Object> paramMap = new HashMap<>();
                                        paramMap.put("width", Integer.valueOf(imgWH.split("x")[0]));
                                        paramMap.put("height", Integer.valueOf(imgWH.split("x")[1]));
                                        blockMap.put("param", paramMap);
                                    }

                                    blockMapList.add(blockMap);
                                }
                            }
                        } else {
                            Map<String, Object> blockMap = Maps.newHashMap();
                            blockMap.put("type", sab.getRealAnswerType());
                            blockMap.put("value", transformedAnswer(sab.getRealAnswerType(), sab.getStudentAnswer()));
                            blockMapList.add(blockMap);
                        }
                    }

                    sectionMap.put("blocks", blockMapList);
                    sectionMapList.add(sectionMap);

                    bodyMap.put("sections", sectionMapList);
                    answerMap.put("body", bodyMap);

                    answerMapList.add(answerMap);
                }

                //将考生结果保存至临时文件
                String answerJson = JsonUtil.toJson(answerMapList);
                String jsonFilePath = tempAnswerDir + "/" + rb.getExamRecordDataId() + ".json";
                IOUtil.toFile(answerJson.getBytes(Charset.forName("UTF-8")), jsonFilePath);
            }

            final String tempAnswerDir = systemConfig.getTempDataDir() + "/exam-" + examId;
            if (!StringUtils.isNullOrEmpty(sb.toString())) {
                String txtFilePath = tempAnswerDir + "/examStudent.txt";
                IOUtil.toFile(sb.toString().getBytes(Charset.forName("UTF-8")), txtFilePath);
            }

        }
    }

    private String transformedAnswer(String answerType, String studentAnswer) {
        if ("image".equals(answerType)) {
            String regExp = "(ftp|https?)\\:\\/\\/([\\w\\_\\-]+)\\.([\\w\\-]+[\\.]?)*[\\w]+\\.[a-zA-Z]{2,10}(.*)\\.(png|jpg|gif|jpeg)";
            return RegExpUtil.find(studentAnswer, regExp);
        }
        return studentAnswer;
    }

    /**
     * 校验答案格式是否正确
     *
     * @param realAnswerType
     * @param studentAnswer
     * @return
     */
    private boolean validateAnswer(String realAnswerType, String studentAnswer) {
        if (StringUtils.isNullOrEmpty(studentAnswer)) {
            return false;
        }

        String regExp;
        switch (realAnswerType) {
            case "image":
                //图片必须包含.png|jpg|gif|jpeg
                regExp = "^(ftp|https?)\\:\\/\\/([\\w\\_\\-]+)\\.([\\w\\-]+[\\.]?)*[\\w]+\\.[a-zA-Z]{2,10}(.*)\\.(png|jpg|gif|jpeg).*$";
                return studentAnswer.matches(regExp);
            case "audio":
                //音频必须包含.mp3
                regExp = "^(ftp|https?)\\:\\/\\/([\\w\\_\\-]+)\\.([\\w\\-]+[\\.]?)*[\\w]+\\.[a-zA-Z]{2,10}(.*)\\.(mp3)";
                return studentAnswer.matches(regExp);
        }

        return true;
    }

    /**
     * 校验字符串和正则表达式是否匹配
     *
     * @param regExp
     * @param input
     * @return
     */
    private boolean isRegExpMatch(String regExp, String input) {
        Pattern pattern = Pattern.compile(regExp);
        Matcher matcher = pattern.matcher(input);
        return matcher.matches();
    }

    public static void main(String[] args) {
        String regExp = "^(ftp|https?)\\:\\/\\/([\\w\\_\\-]+)\\.([\\w\\-]+[\\.]?)*[\\w]+\\.[a-zA-Z]{2,10}(.*)\\.(png|jpg|gif|jpeg)(\\s|\\S)*$";
        String regExp1 = "(ftp|https?)\\:\\/\\/([\\w\\_\\-]+)\\.([\\w\\-]+[\\.]?)*[\\w]+\\.[a-zA-Z]{2,10}(.*)\\.(png|jpg|gif|jpeg)";
        String regExp2 = "(\\d+)x(\\d+)";
        String input = "https://ecs-test-static.qmth.com.cn/oe-answer-file/3/7/20/3_7_20_15748449608356000.jpeg!/both/200x200";
        Pattern pattern = Pattern.compile(regExp);
        Matcher matcher = pattern.matcher(input);
//
        System.out.println(matcher.matches());
        System.out.println(matcher.group());
        System.out.println(RegExpUtil.find(input, regExp1));
        System.out.println(RegExpUtil.find(input, regExp2));

    }
}
